function ControlRoom() {

  /**
   * Init whole Control Room
   * @returns {Promise<void>}
   */
  let init = async function () {

    await renderAll();

    $('#controlRoomClose').click(hideControlRoomCallback);

  }

  /**
   * Render all components for controlRoom
   */
  async function renderAll() {
    const services = await createListOfServices();
    const videoChannelId = '#controlRoom-VideoChannel';
    const resolutionId = '#controlRoom-Resolution';
    const imagePreviewId = 'imagePreviewCamera2';
    const liveViewId = '#controlRoom-LiveView';
    let liveViewEnabled = true;

    // Create video channel
    await createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].camera);
    if (isMips()) {
      $(videoChannelId).hide();
    }

    // Create resolution list, depend on video channel
    let bestResolution = getBestResolution(services);
    await createResolutionList(resolutionId);
    $(resolutionId + ' option:contains(' + bestResolution + ')').attr('selected', 'selected');

    // Init checkbox live view enabled
    $(liveViewId).prop('checked', liveViewEnabled);

    // Refresh preview
    refreshImagePreview();

    // Register events
    $(liveViewId).off('change').on('change', function () {
      liveViewEnabled = $(this).prop('checked');
      if (liveViewEnabled) {
        refreshImagePreview();
      } else {
        hideImageFromCamera(imagePreviewId);
      }
    });
    $(videoChannelId).off('change').on('change', function () {
      let selectedCamera = saveVideoChannelValue(serviceArrTmp[activeItem].camera, $(this));
      selectedCamera = selectedCamera < 0 ? 0 : selectedCamera;
      cameraResolutions = createNewResolutionList(params['properties.image.i' + selectedCamera + '.resolution'] || params['properties.image.resolution'])
      resolution = createResolutionObject(params['image.i' + selectedCamera + '.appearance.resolution']);
      createResolutionList(resolutionId);
      if (liveViewEnabled) {
        refreshImagePreview();
      }
    });
    $(resolutionId).off('change').on('change', function () {
      if (liveViewEnabled) {
        refreshImagePreview();
      }
    });

    let cameraSupportsPtz = await supportsPtz();
    if (cameraSupportsPtz) {

      let ptzPositionListJqueryObject = $('#ptzPositionList');
      ptzPositionListJqueryObject.showSpinner();
      let ptzPositionList = await ptzManager.getList(serviceArrTmp[activeItem].camera);

      if (ptzPositionList.length === 0) {
        // If camera has empty view areas, dont load PTZ
        ptzPositionListJqueryObject.hideSpinner();
      } else {
        let ptzCurrentPosition = await ptzManager.getCurrentPosition();
        let ptzPositionHTML = await createPtzPositionHTML(ptzPositionList, ptzCurrentPosition);
        ptzPositionListJqueryObject.html('').append(ptzPositionHTML);

        $('.setPosition').off('click').on('click', async function () {
          $('.setPosition.btn-primary').addClass('btn-secondary').removeClass('btn-primary');
          $(this).addClass('btn-primary');
          let position_name = $(this).attr('data-position-name');
          await ptzManager.setPosition(position_name, serviceArrTmp[activeItem].camera);
        });
      }

    }

    // Check if is more services than it's allowed
    const exceeds_max_service_count = await configurationTmp.exceedsMaxServiceCount();
    if (exceeds_max_service_count) {
      const init_alert_message = await configurationTmp.getMaxAllowedMessage();
      showMessage('warning', init_alert_message, false, 'maxAllowedServices');
    }

    let controlRoomList = $('#controlRoomList');
    controlRoomList.html(''); // Empty list

    // Fill list and attach JS events on each service
    for (let i = 0; i < services.length; i++) {
      const allowed_control = ['manual', 'time'].includes(services[i].automationType);
      if (allowed_control) {
        const service_for_render = renderService(services[i]);
        controlRoomList.append(service_for_render); // Append rendered service to list
        attachScriptToService(services[i]); // Attach events to service
      }
    }

    // After change text value, change textarea with new text
    $('.controlRoomFieldSelect').off('change').on('change', function () {
      var services = createListOfServices();
      var tabName = $(this).attr('data-tab-name');
      var serviceID = parseInt($(this).attr('data-id'));
      var newVal = parseInt($(this).val());
      for (var i = 0; i < services.length; i++) {
        var service = services[i];
        if (service.name === 'textAndBackground' || service.name === 'customGraphics') {
          for (var j = 0; j < service.mappingZones.length; j++) {
            if (service.mappingZones[j].name === tabName && service.id === serviceID) {
              let textareaElementId = $('#textAndBackgroundText-' + serviceID + '-' + tabName + '-InputText');
              let textValue = service.mappingZones[j].text[newVal].source;
              setHeightOfTextarea(textareaElementId, textValue);
              textareaElementId.val(textValue);
            }
          }
        }
      }
    });

    // Event for service actions (show, hide, update)
    $('.serviceAction').off('click').on('click', function (e) {
      e.preventDefault();
      onServiceActionClick($(this));
    });

    Sortable.create(controlRoomList[0], {
      handle: '.serviceElementHandle',
      dataIdAttr: 'data-id',
      animation: 150,
      ghostClass: 'draggableElementShadow',
      store: {
        set: function (sortable) {
          saveNewOrderOfServices(sortable.toArray());
        },
      }
    });

    function refreshImagePreview() {
      let videoChannelVal = $(videoChannelId).val();
      let resolutionVal = $(resolutionId).val();
      showImageFromCamera(imagePreviewId, cameraResolutions[resolutionVal], videoChannelVal);
    }
  }

  function createPtzPositionHTML(list, currPosition) {
    let html_output = ['<span class="p-1 mr-1">PTZ positions:</span>'];

    list.forEach(item => {
      let active = (item === currPosition) ? 'btn-primary' : 'btn-secondary';
      let html_item = '<button class="btn ' + active + ' btn-sm mr-1 setPosition" data-position-name="' + item + '">' + item + '</button>'
      html_output.push(html_item);
    });

    return html_output.join('');
  }

  function saveNewOrderOfServices(order) {
    let newServiceArrTmp = [];
    order.forEach(function (item, i) {
      serviceArrTmp.forEach(function (service) {
        if (service.id === parseInt(item)) {
          newServiceArrTmp.push(service);
        }
      });
    });
    serviceArrTmp = [];
    serviceArrTmp = newServiceArrTmp;
    saveBtnCallback(true, newServiceArrTmp);
  }

  /**
   * Get most common and biggest resolution
   * @param {Array} services - array of services
   * @returns {string}
   */
  function getBestResolution(services) {
    // Get resolution from service arr
    let resArr = [];
    services.forEach(function (service) {
      if (service.enabled === 1) {
        resArr.push(service.width + " x " + service.height);
      }
    });

    // Count occurrences of resolution
    let occurrencesArr = [];
    resArr.forEach(function (item) {
      occurrencesArr[item] = occurrencesArr[item] ? ++occurrencesArr[item] : 1;
    });

    // Get max value from occurrencesArr
    let maxValue = Object.values(occurrencesArr).sort().pop();

    // Get all max occurrences
    let allMaxValues = Object.keys(occurrencesArr).filter(key => occurrencesArr[key] === maxValue);

    // Get max width from resolution
    let maxWidth = 0;
    for (let i = 0; i < allMaxValues.length; i++) {
      let currWidth = parseInt((allMaxValues[i].split("x"))[0]);
      maxWidth = maxWidth < currWidth ? currWidth : maxWidth;
    }

    // Get max resolution from arr and return
    return allMaxValues.find(item => item.includes(maxWidth));
  }

  /**
   * Events for service action (show & update, hide, update)
   * @param element - $(this)
   */
  const onServiceActionClick = async (element) => {
    let show_and_update = element.hasClass('service_ShowAndUpdate');
    let hide = element.hasClass('service_Hide');
    let update = element.hasClass('service_Update');
    const serviceID = parseInt(element.closest('.serviceElement').attr('data-id'));

    if (show_and_update) {
      const can_enable_another_service = await configurationTmp.canEnableAnotherService();
      if (!can_enable_another_service) {
        const alert_message = await configurationTmp.getMaxAllowedMessage();
        showMessage('warning', alert_message, false, 'maxAllowedServices');
      } else {
        handleShowAndUpdate(serviceID);
      }
    } else if (hide) {
      handleHideService(serviceID);
    } else if (update) {
      handleUpdateService(serviceID);
    }

  }

  /**
   * @param {number} serviceID
   */
  const handleUpdateService = (serviceID) => {
    for (let i = 0; i < serviceArrTmp.length; i++) {
      if (serviceArrTmp[i].id === serviceID) {
        serviceArrTmp[i] = JSON.parse(JSON.stringify(saveService(serviceArrTmp[i])));
      }
    }
    saveBtnCallback(false);
  }

  /**
   * @param {number} serviceID
   */
  const handleShowAndUpdate = (serviceID) => {
    const serviceElement = 'div.serviceElement[data-id="' + serviceID + '"]';
    const collapseElement = '#itemBody' + serviceID;
    $(collapseElement).collapse('show');
    $(serviceElement).find('.service_ShowAndUpdate').hide();
    $(serviceElement).find('.service_Hide').show();

    for (let i = 0; i < serviceArrTmp.length; i++) {
      if (serviceArrTmp[i].id === serviceID) {
        // 1. Show
        serviceArrTmp[i].enabled = 1;
        // 2. Update
        serviceArrTmp[i] = JSON.parse(JSON.stringify(saveService(serviceArrTmp[i])));
      }
    }

    saveBtnCallback(true);
  }

  /**
   * @param {number} serviceID
   */
  const handleHideService = (serviceID) => {
    const serviceElement = 'div.serviceElement[data-id="' + serviceID + '"]';
    $(serviceElement).find('.service_Hide').hide();
    $(serviceElement).find('.service_ShowAndUpdate').show();

    for (let i = 0; i < serviceArrTmp.length; i++) {
      if (serviceArrTmp[i].id === serviceID) {
        serviceArrTmp[i].enabled = 0;
      }
    }
    saveBtnCallback(true);
  }

  /** Attach JS script to service
   * @param service - JSON object
   */
  function attachScriptToService(service) {
    var serviceID = service.id;

    if (service.name === 'textAndBackground' || service.name === 'customGraphics') {

      var mappingArea = service.background;
      var mappingAreaElement = $('#textAndBackgroundMappingArea' + serviceID);
      var inputGroupImage = $('#textAndBackgroundImage' + serviceID).closest('.input-group');
      mappingAreaElement.val(mappingArea);
      (mappingArea === 'image') ? inputGroupImage.removeClass('d-none') : inputGroupImage.addClass('d-none');
      // After change mapping area select, show or hide <select> with image list
      mappingAreaElement.on('change', function () {
        mappingArea = $('#textAndBackgroundMappingArea' + serviceID).val();
        (mappingArea === 'image') ? inputGroupImage.removeClass('d-none') : inputGroupImage.addClass('d-none');
      });
      // Set all fields
      for (var i = 0; i < service.mappingZones.length; i++) {
        var selectElementId = $('#textAndBackgroundSelect-' + serviceID + "-" + service.mappingZones[i].name);
        var textareaElementId = $('#textAndBackgroundText-' + serviceID + "-" + service.mappingZones[i].name + '-InputText');
        // Fill select element with options
        var selected = 0;
        var optionList = '';
        for (var o = 0; o < service.mappingZones[i].text.length; o++) {
          if (service.mappingZones[i].text[o].active) selected = o;
          optionList += '<option value="' + o + '">text ' + (o + 1) + '</option>';
        }
        selectElementId
            .html(optionList)
            .val(selected);
        // Fill textarea with value and resize
        var selectValue = selectElementId.val();
        var textValue = service.mappingZones[i].text[parseInt(selectValue)].source;
        setHeightOfTextarea(textareaElementId, textValue);
        textareaElementId.text(textValue);
      }
      // Create list of option for <select>
      createImageList('#textAndBackgroundImage' + serviceID, false, service.image);
    } else if (service.name === 'infoticker') {
      // Resize textarea
      let infotickerSourceElementId = $('#infotickerSource' + service.id);
      let textValue = infotickerSourceElementId.val();
      setHeightOfTextarea(infotickerSourceElementId, textValue);
    } else if (service.name === 'images') {

      const custom_overlay_btns = $('.custom_overlay_set_active');
      custom_overlay_btns.click((event) => {
        const active_id = Number($(event.target).attr('data-id'));
        custom_overlay_btns.each(
          (index, item) => {
            const is_active = active_id === index;
            ButtonControl(item).setActive(is_active);
            handleSaveOverlay(item, is_active);
          }
        );
        saveBtnCallback(true);
      });
    }

    $('[title]').tooltip();
  }

  const ButtonControl = (item) => {
    const setActive = (is_active) => {
      const item_jquery_object = $(item);
      is_active ? item_jquery_object.removeClass('btn-secondary').addClass('btn-primary')
        : item_jquery_object.removeClass('btn-primary').addClass('btn-secondary');
    }
    return {
      setActive,
    }
  }

  /**
   * @param textareaElement - textarea jquery element
   * @param textValue - value of textarea
   */
  function setHeightOfTextarea(textareaElement, textValue) {
    let numberOfLines = (textValue.split("\n")).length;
    const charsPerLine = 85; // 85 chars per line
    if (numberOfLines === 1) {
      numberOfLines = Math.ceil((textValue.length * 1.1) / charsPerLine);
    }
    if (textValue.length === 0) {
      numberOfLines = 1;
    }
    if (numberOfLines > 5) { // Max height of textarea is 5 lines
      numberOfLines = 5;
    }
    textareaElement.attr('rows', numberOfLines);
  }

  /** Save service
   * @param service - JSON object
   * @return JSON object
   */
  function saveService(service) {
    if (service.name === 'textAndBackground' || service.name === 'customGraphics') {

      service.background = $('#textAndBackgroundMappingArea' + service.id).val();
      service.image = $('#textAndBackgroundImage' + service.id).val();

      var selects = $('[id^=textAndBackgroundSelect-' + service.id + ']');
      var textareas = $('[id^=textAndBackgroundText-' + service.id + '-]');
      for (var i = 0; i < selects.length; i++) {
        for (var o = 0; o < service.mappingZones[i].text.length; o++) {
          if (o !== parseInt($(selects[i]).val())) {
            service.mappingZones[i].text[o].active = false; // Unset previous active
          } else {
            service.mappingZones[i].text[o].active = true; // Set new active
            service.mappingZones[i].text[o].source = $(textareas[i]).val(); // Set new text value
          }
        }
      }

    } else if (service.name === 'infoticker') {
      service.source = $('#infotickerSource' + service.id).val();
    }
    return service;
  }

  /** Create HTML Service
   * @param service - JSON object
   * @return elementService - string, contains HTML of one service
   */
  function renderService(service) {
    /** In service 'custom|images' display list of image in collapsed state
     * customList is list of images in service 'custom|images'
     * customClass has empty value or 'custom|images', it helps render service with class .custom height than other services
     */
    let isCustom = ['images', 'custom'].includes(service.name);
    let customList = isCustom ? '<div style="height: auto; width: 100%;">' + getCustomOverlay(service) + '</div>' : '';
    let customClass = isCustom ? service.name : '';

    // Define service and options
    let elementService = '<div id="serviceElement' + service.id + '" data-name="' + service.name + '" data-id="' + service.id + '" class="m-2 p-0 p-sm-2 mt-0 mb-2 serviceElement border-bottom pb-3 ' + customClass + '">';
    let showUpdateBtn = ['infoticker', 'textAndBackground', 'customGraphics'].includes(service.name);

    let updateBtn = (showUpdateBtn) ? '<a href="" class="ml-2 btn btn-sm btn-info service_Update serviceAction">Update</a>' : '';
    let displayShow = (service.enabled) ? 'style="display:none"' : '';
    let displayHide = (!service.enabled) ? 'style="display:none"' : '';
    let showBtn = '<a href="" class="ml-2 btn btn-sm btn-success service_ShowAndUpdate serviceAction" ' + displayShow + '>Show & Update</a>';
    let hideBtn = '<a href="" class="ml-2 btn btn-sm btn-danger service_Hide serviceAction" ' + displayHide + '>Hide</a>';
    let serviceNameFull = service.customName ? escapeHtml(service.customName) : getFullNameOfService(service.name);
    let running = (service.enabled) ? 'true' : 'false';
    let show = (service.enabled) ? 'show' : '';
    let inputs = renderInputs(service);

    let itemHeader = '<div class="serviceHeader">';
    itemHeader += '<button class="serviceElementHandle mr-2 btn btn-sm btn-link"><span class="fa fa-grip-vertical"></span></button>'; //Grab handler
    if (inputs !== "") { // Link for collapsing only if inputs are not empty
      itemHeader += '<a class="serviceName btn btn-link mr-2" data-toggle="collapse" data-target="#itemBody' + service.id + '" role="button" aria-expanded="' + running + '" aria-controls="itemBody' + service.id + '">';
    } else {
      itemHeader += '<a class="serviceName containerText btn btn-link mr-2" style="cursor:default;">';
    }
    itemHeader += '<div class="serviceNameId font-weight-bold" data-name="' + serviceNameFull + '" data-id="' + service.id + '">' + serviceNameFull + '</div><div class="ml-1 text-muted">(ID: ' + service.id + ')</div>'; // Service name with ID
    itemHeader += (inputs !== "") ? '<div class="ml-2 fas fa-chevron-down iconPlus"></div><div class="ml-2 fas fa-chevron-up iconMinus"></div>' : ''; // Arrows for collapsing
    itemHeader += '</a>';
    itemHeader += customList + '</div>';

    let innerDivInBody = (inputs !== "") ? '<div style="padding-bottom: 40px;">' + inputs + '</div>' : '<div>' + inputs + '</div>';
    let itemBody = '<div class="collapse ' + show + '" id="itemBody' + service.id + '">' + innerDivInBody + '</div>';

    let itemFooter = '<div class="serviceFooter"><div class="serviceBtns text-right mt-1 mr-3">' + updateBtn + showBtn + hideBtn + '</div></div>';

    elementService += itemHeader + itemBody + itemFooter;
    elementService += '</div>';

    return elementService;
  }

  /** Render different types of inputs for each service
   * @param service is json object
   * @return string of HTML with inputs
   */
  function renderInputs(service) {
    var inputs = '';
    if (service.name === 'textAndBackground' || service.name === 'customGraphics') {
      inputs = getTextAndBackground(service);
    } else if (service.name === 'infoticker') {
      inputs = (service.sourceType === 'text') ? getInfoTicker(service) : '';
    } //missing service 'custom' because it's rendered in renderService
    return inputs;
  }

  /** Create HTML inputs for Custom Overlay
   * if triggering - automatic - display list of images
   * if triggering - manual - display list of btns for control active overlay
   * @param service is json object
   * @return string of HTML for service 'customOverlay'
   */
  function getCustomOverlay(service) {
    let default_html_automatic = '<span class="text-muted" style="padding-left: 5px; padding-right: 200px; white-space: nowrap;text-overflow: ellipsis;width: 100%;display: block;overflow: hidden;">%REPLACE%</span>';
    let default_html_manual = '<div>%REPLACE%</div>';
    let items = [];
    let overlayList = service.overlayList;
    let is_manual = service.automationType === 'manual';
    for (let i = 0; i < overlayList.length; i++) {
      if (is_manual) {
        let img_name = handle_image_name(overlayList[i].imgName);
        let btn_classes = overlayList[i].active ? 'btn-primary' : 'btn-secondary';
        const btn_html = '<btn class="custom_overlay_set_active btn btn-sm ' + btn_classes + ' ml-1 mb-1" data-id="' + i + '">' + img_name + '</btn>';
        items.push(btn_html);
      } else {
        items.push(overlayList[i].imgName);
      }
    }
    return is_manual ? default_html_manual.replace('%REPLACE%', items.join('')) : default_html_automatic.replace('%REPLACE%', items.join(', '));
  }

  const handle_image_name = (img_name) => {
    if (img_name.length > 15) {
      const file_type = img_name.split(".")[1];
      img_name = img_name.slice(0, 15) + '...' + file_type;
    }
    return img_name;
  }


  /** Create HTML inputs for Info Ticker
   * @param service is json object
   * @return string of HTML for service 'infoTicker'
   */
  function getInfoTicker(service) {
    var all = $('#patternServices > #infoTicker').html();
    let serviceSource = replaceSpecialChars(service.source);
    all = all.replace(/%ID%/g, service.id);
    all = all.replace(/%VALUE%/g, serviceSource);
    return all;
  }

  /** Create HTML inputs for textAndBackground
   * @param service is json object
   * @return string of HTML for service 'infoTicker'
   */
  function getTextAndBackground(service) {
    var all = $('#patternServices > #textAndBackground').html();
    var allFields = '';

    allFields += '<div class="col">';
    for (var i = 0; i < service.mappingZones.length; i++) {
      let mappingZoneName = replaceSpecialChars(service.mappingZones[i].name);
      var textLabel = '<div for="textAndBackgroundText%TAB_NAME%InputText" class="input-group-prepend"><span class="input-group-text">' + mappingZoneName + '</span></div>';
      var selectSource = '<select class="form-control controlRoomFieldSelect" id="textAndBackgroundSelect-%ID%-%TAB_NAME%" data-id="%ID%" data-tab-name="%TAB_NAME%"></select>';
      var textField = '<textarea class="form-control controlRoomFieldTextarea" rows="1" id="textAndBackgroundText-%ID%-%TAB_NAME%-InputText" placeholder="Use enter to break the lines"></textarea>';
      var inputGroup = '<div class="input-group mt-1">'
          + textLabel.replace(/%TAB_NAME%/g, mappingZoneName)
          + selectSource.replace(/%TAB_NAME%/g, mappingZoneName)
          + textField.replace(/%TAB_NAME%/g, mappingZoneName)
          + '</div>';
      allFields += inputGroup;
    }
    allFields += '</div>';

    all += allFields;
    all = all.replace(/%ID%/g, service.id);
    return all;
  }

  /** Helper fce for service name
   * @param serviceName - string
   * @return serviceName - string, nice name of service
   */
  function getFullNameOfService(serviceName) {
    if (serviceName === 'accuweather')
      serviceName = 'Weather (AccuWeather)';
    else if (serviceName === 'custom' || serviceName === 'images')
      serviceName = 'Images';
    else if (serviceName === 'ptz')
      serviceName = 'PTZ Overlay';
    else if (serviceName === 'ptzCompass')
      serviceName = 'PTZ Compass';
    else if (serviceName === 'infoticker')
      serviceName = 'Info Ticker';
    else if (serviceName === 'instant')
      serviceName = 'Instant Overlay';
    else if (serviceName === 'sportzcast')
      serviceName = 'Scoreboard (Sportzcast)';
    else if (serviceName === 'textAndBackground' || serviceName === 'customGraphics')
      serviceName = 'Custom Graphics';
    else if (serviceName === 'pip')
      serviceName = 'Picture In Picture';
    else if (serviceName === 'scripter')
      serviceName = 'Scripter';
    return serviceName;
  }

  /** Create array of services
   * @return {Array} - array of services
   */
  function createListOfServices() {
    var services = [];
    for (var i = 0; i < serviceArrTmp.length; i++) {
      services.push(serviceArrTmp[i]);
    }
    return services;
  }

  return {
    init,
  }

}